//! ## Path data guesswork for BIP-0032 derivation paths.

#![allow(clippy::unreadable_literal)]

use keyfork_derive_util::{DerivationIndex, DerivationPath};

/// The default derivation path for OpenPGP.
pub static OPENPGP: DerivationIndex = DerivationIndex::new_unchecked(7366512, true);

/// A derivation target.
pub enum Target {
    /// An OpenPGP key, whose account is the given index.
    OpenPGP(DerivationIndex),
}

impl std::fmt::Display for Target {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Self::OpenPGP(account) => {
                write!(f, "OpenPGP key (account {account})")
            }
        }
    }
}

/// Determine the closest [`Target`] for the given path. This method is intended to be used by
/// `keyforkd` to provide an optional textual prompt to what a client is attempting to derive.
pub fn guess_target(path: &DerivationPath) -> Option<Target> {
    Some(match path.iter().collect::<Vec<_>>()[..] {
        [t, index] if t == &OPENPGP => Target::OpenPGP(index.clone()),
        _ => return None,
    })
}
